@@ -10,7 +10,7 @@
 --
 --      guichan.lua - Define the main guichan menu.
 --
---      (c) Copyright 2006-2015 by Jimmy Salmon, Pali Rohár and cybermind
+--      (c) Copyright 2006-2016 by Jimmy Salmon, Pali Rohár and cybermind
 --
 --      This program is free software; you can redistribute it and/or modify
 --      it under the terms of the GNU General Public License as published by
@@ -835,6 +835,32 @@ function RunMap(map, objective, fow, revealmap, results)
   SetDefaultRaceView()
 end
 
+function RunDemo()
+  Objectives = DefaultObjectives
+  local mapname = "maps/demo/demo0" .. math.random(1, 4) .. ".smp"
+  InitGameVariables()
+  SetFogOfWar(false)
+  RevealMap()
+  OldShowTips = wc2.preferences.ShowTips
+  wc2.preferences.ShowTips = false
+  OldSinglePlayerTriggers = SinglePlayerTriggers
+  function SinglePlayerTriggers()
+    AddTrigger(
+     function() return GameCycle > 1000 end,
+     function() return ActionDraw() end)
+  end
+  Load(mapname)
+  GameSettings.Presets[0].Team = 1
+  GameSettings.Presets[1].Team = 2
+  GameSettings.Presets[2].Type = 3
+  GameSettings.Presets[2].Race = 2
+  StartMap(mapname)
+  wc2.preferences.ShowTips = OldShowTips
+  SinglePlayerTriggers = OldSinglePlayerTriggers
+  InitGameSettings()
+  SetDefaultRaceView()
+end
+
 function SetDefaultPlayerNames()
 -- Add player names according to player color
 	for i=0,7 do
@@ -1066,7 +1092,7 @@ function RunSinglePlayerGameMenu()
   local tileset_names = {_("Map Default"), _("Summer"), _("Swamp"), _("Wasteland"), _("Winter")}
   local numunit_types = {_("Map Default"), _("One Peasant Only")}
   local difficulty_types = {_("Easy"), _("Normal"), _("Hard"),_("Nightmare"),_("Hell")}
-  local resource_types = {_("Map Default"), _("High"), _("Medium"), _("Low"),_("Quick Start")}
+  local resource_types = {_("Map Default"), _("Low"), _("Medium"), _("High"),_("Quick Start")}
   local nms = {nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil}
   local pcolor = {nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil}
   local ptype = {nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil}
@@ -1533,6 +1559,19 @@ function BuildProgramStartMenu()
 
   menu:addFullButton(_("E~!xit Program"), "x", offx + 208, offy + 104 + 36*7,
     function() menu:stop() end)
+
+  local time = 0
+  function checkRunDemo()
+    time = time +1
+    if (time > 2000) then
+      time = 0
+      RunDemo()
+      wargus.playlist = { "music/Orc Briefing" .. wargus.music_extension }
+      PlayMusic("music/Orc Briefing" .. wargus.music_extension)
+    end
+  end
+  local listener = LuaActionListener(function(s) checkRunDemo() end)
+  menu:addLogicCallback(listener)
 	
   return menu:run()
 end
