@@ -112,7 +112,7 @@ DefineUnitType("unit-attack-peasant", { Name = _("Minuteman"),
   Image = {"file", "human/units/peasant.png", "size", {72, 72}},
   DrawLevel = 19,
   Animations = "animations-peasant", Icon = "icon-peasant",
-  Costs = {"time", 45, "gold", 400},
+  Costs = {"time", 0, "gold", 0},
   Speed = 10,
   HitPoints = 30,
   DrawLevel = 40,
@@ -158,6 +158,7 @@ DefineUnitType("unit-ballista", { Name = _("Ballista"),
   RightMouseAction = "attack",
   CanAttack = true,
   CanTargetLand = true, CanTargetSea = true,
+  GroundAttack = true,
   SelectableByRectangle = true,
   Sounds = {
     "selected", "ballista-selected",
@@ -186,6 +187,7 @@ DefineUnitType("unit-ballista-super", { Name = _("Ballista"),
   RightMouseAction = "attack",
   CanAttack = true,
   CanTargetLand = true, CanTargetSea = true, CanTargetAir = true,
+  GroundAttack = true,
   SelectableByRectangle = false,
   Sounds = {
     "selected", "ballista-selected",
@@ -537,6 +539,9 @@ DefineUnitType("unit-human-oil-tanker", { Name = _("Oil tanker"),
   Sounds = {
     "selected", "human-oil-tanker-selected",
     "acknowledge", "human-oil-tanker-acknowledge",
+    -- the oil tankers do not use the nasal "work's done" peasant sound for
+    -- completing buildings
+    "work-complete", "basic human voices research complete",
     "ready", "human-oil-tanker-ready",
     "help", "basic human voices help 1",
     "dead", "ship sinking"} } )
@@ -620,6 +625,7 @@ DefineUnitType("unit-battleship", { Name = _("Battleship"),
   SideAttack = true,
   CanAttack = true,
   CanTargetLand = true, CanTargetSea = true,
+  GroundAttack = true,
   SeaUnit = true,
   SelectableByRectangle = true,
   Sounds = {
@@ -684,6 +690,7 @@ DefineUnitType("unit-balloon", { Name = _("Gnomish Flying Machine"),
   AirUnit = true, Coward = true,
   DetectCloak = true,
   SelectableByRectangle = true,
+  OnReady = AiExploreUnit,
   Sounds = {
     "selected", "gnomish-flying-machine-selected",
     "acknowledge", "gnomish-flying-machine-acknowledge",
@@ -763,7 +770,7 @@ DefineUnitType("unit-wise-man", { Name = _("Lothar"),
   SightRange = 5, ComputerReactionRange = 7, PersonReactionRange = 5,
   Armor = 4, BasicDamage = 8, PiercingDamage = 4, Missile = "missile-none",
   MaxAttackRange = 1,
-  Priority = 65,
+  Priority = 65, AnnoyComputerFactor = 70,
   Points = 120,
   Demand = 1,
   Corpse = "unit-human-dead-body",
