@@ -10,7 +10,7 @@
 --
 --      network.lua - Define the menu for network game.
 --
---      (c) Copyright 2005-2015 by François Beerten, Jimmy Salmon, Pali Rohár
+--      (c) Copyright 2005-2016 by François Beerten, Jimmy Salmon, Pali Rohár
 --								   and Cybermind.
 --
 --      This program is free software; you can redistribute it and/or modify
@@ -50,7 +50,7 @@ function ErrorMenu(errmsg)
   menu:setPosition((Video.Width - 288) / 2, (Video.Height - 128) / 2)
   menu:setDrawMenusUnder(true)
 
-  menu:addLabel("Error:", 144, 11)
+  menu:addLabel(_("Error:"), 144, 11)
 
   local l = MultiLineLabel(errmsg)
   l:setFont(Fonts["large"])
@@ -67,20 +67,63 @@ function ErrorMenu(errmsg)
   menu:run()
 end
 
-function addPlayersList(menu, numplayers)
+
+
+function addPlayersList(menu, numplayers, isserver)
   local i
   local players_name = {}
   local players_state = {}
   local sx = Video.Width / 20
   local sy = Video.Height / 20
   local numplayers_text
+  local ainumber
+  local ainumberCb
+  local ainumberlist = {}
+  local updateAInumberList
+  local requestedNumberOfAI
+
+  menu:writeText(_("AIs:"), sx, sy*11+75)
+
+  if isserver then
+    updateAInumberList = function(connected_players)
+      -- create list with number of available slots for computer players
+      ainumberlist = {}
+      local maxAIplayers = numplayers - 1 - connected_players
+      for i=0,maxAIplayers do
+        table.insert(ainumberlist, tostring(i) .. _(" AI player(s)"))
+      end
+    end
+
+    updateAInumberList(0)
+    ainumber = menu:addDropDown(ainumberlist, sx + 100, sy*11+75, function() end)
+    requestedNumberOfAI = function()
+      -- parse the requested number of AI players as integer
+      return tonumber(string.gmatch(ainumberlist[ainumber:getSelected() + 1], "%d+")())
+    end
+    ainumberCb = function()
+      ServerSetupState.Opponents = requestedNumberOfAI()
+      GameSettings.Opponents = ServerSetupState.Opponents
+      NetworkServerResyncClients()
+    end
+    ainumber:setActionCallback(ainumberCb)
+    ainumber:setSize(190, 20)
+    ainumber:setSelected(GameSettings.Opponents)
+  else
+    ainumber = menu:writeText(tostring(ServerSetupState.Opponents), sx + 100, sy*11+75)
+    ainumberCb = function()
+      ainumber:setCaption(tostring(ServerSetupState.Opponents))
+      ainumber:adjustSize()
+    end
+  end
 
   menu:writeLargeText(_("Players"), sx * 11, sy*3)
   for i=1,8 do
     players_name[i] = menu:writeText(_("Player")..i, sx * 11, sy*4 + i*18)
     players_state[i] = menu:writeText(_("Preparing"), sx * 11 + 80, sy*4 + i*18)
   end
-  numplayers_text = menu:writeText(_("Open slots : ") .. numplayers - 1, sx *11, sy*4 + 144)
+  if isserver then
+    numplayers_text = menu:writeText(_("Open slots : ") .. numplayers - 1, sx *11, sy*4 + 144)
+  end
 
   local function updatePlayers()
     local connected_players = 0
@@ -93,8 +136,8 @@ function addPlayersList(menu, numplayers)
         players_name[i]:setCaption("")
         players_state[i]:setCaption("")
       else
-        connected_players = connected_players + 1
-        if ServerSetupState.Ready[i-1] == 1 then
+	connected_players = connected_players + 1
+	if ServerSetupState.Ready[i-1] == 1 then
           ready_players = ready_players + 1
           players_state[i]:setCaption(_("Ready"))
         else
@@ -104,9 +147,26 @@ function addPlayersList(menu, numplayers)
         players_name[i]:adjustSize()
       end
     end
-    numplayers_text:setCaption(_("Open slots : ") .. numplayers - 1 - connected_players)
-    numplayers_text:adjustSize()
-    return numplayers == 1 or (connected_players > 0 and ready_players == connected_players)
+    local currentAInumber = 0
+    if isserver then
+      updateAInumberList(connected_players)
+      currentAInumber = requestedNumberOfAI()
+      ainumber:setList(ainumberlist)
+      ainumber:setSelected(currentAInumber)
+    else
+      ainumberCb()
+    end
+    if isserver then
+      local openSlots = numplayers - 1 - connected_players
+      openSlots = openSlots - ServerSetupState.Opponents
+      numplayers_text:setCaption(_("Open slots : ") .. openSlots)
+      numplayers_text:adjustSize()
+    end
+
+    -- only 1 player in this map or
+    -- all connected players are ready or
+    -- we could play against AI
+    return numplayers == 1 or (connected_players > 0 and ready_players == connected_players) or (connected_players == 0 and currentAInumber ~= 0)
   end
 
   return updatePlayers
@@ -134,8 +194,8 @@ function RunJoiningMapMenu(optRace, optReady)
   local fow = menu:addImageCheckBox(_("Fog of war"), sx, sy*3+120, offi, offi2, oni, oni2, function() end)
   fow:setMarked(true)
   ServerSetupState.FogOfWar = 1
-  ServerSetupState.Opponents = 15
-  GameSettings.Opponents = 15
+  ServerSetupState.Opponents = 0
+  GameSettings.Opponents = 0
   fow:setEnabled(false)
   local revealmap = menu:addImageCheckBox(_("Reveal map"), sx, sy*3+150, offi, offi2, oni, oni2, function() end)
   revealmap:setEnabled(false)
@@ -189,7 +249,7 @@ function RunJoiningMapMenu(optRace, optReady)
   end
   local readycheckbox = menu:addImageCheckBox(_("~!Ready"), sx*11, sy*14, offi, offi2, oni, oni2, readycb)
 
-  local updatePlayersList = addPlayersList(menu, numplayers)
+  local updatePlayersList = addPlayersList(menu, numplayers, false)
 
   local joincounter = 0
   local delay = 4
@@ -203,6 +263,7 @@ function RunJoiningMapMenu(optRace, optReady)
     GameSettings.NumUnits = ServerSetupState.UnitsOption
     resources:setSelected(ServerSetupState.ResourcesOption)
     GameSettings.Resources = ServerSetupState.ResourcesOption
+    GameSettings.Opponents = ServerSetupState.Opponents
     updatePlayersList()
     state = GetNetworkState()
     -- FIXME: don't use numbers
@@ -231,12 +292,14 @@ function RunJoiningMapMenu(optRace, optReady)
         end
         NetworkGamePrepareGameSettings()
         RunMap(NetworkMapName)
-        PresentMap = OldPresentMap
+	PresentMap = OldPresentMap
         DefinePlayerTypes = oldDefinePlayerTypes
+        MetaClient:Close()
         menu:stop()
       end
     elseif (state == 10) then -- ccs_unreachable
       ErrorMenu(_("Cannot reach server"))
+      MetaClient:Close()
       menu:stop()
     end
   end
@@ -249,12 +312,12 @@ function RunJoiningMapMenu(optRace, optReady)
   end
 
   menu:addFullButton(_("~!Cancel"), "c", Video.Width / 2 - 100, Video.Height - 100,
-    function() NetworkDetachFromServer(); menu:stop() end)
+    function() NetworkDetachFromServer(); MetaClient:Close(); menu:stop() end)
 
   menu:run()
 end
 
-function RunJoiningGameMenu(optRace, optReady)
+function RunJoiningGameMenu(optRace, optReady, optExtraLabel, optStopDirect)
   local menu = nil
   if (optRace and optReady) then
     menu = WarMenu(_("Joining Game"))
@@ -265,7 +328,11 @@ function RunJoiningGameMenu(optRace, optReady)
     menu:setDrawMenusUnder(true)
   end
 
-  menu:addLabel(_("Connecting to server"), 144, 11)
+  if optExtraLabel then
+    menu:addLabel(optExtraLabel, 144, 11)
+  else
+    menu:addLabel(_("Connecting to server"), 144, 11)
+  end
 
   local percent = 0
 
@@ -284,13 +351,21 @@ function RunJoiningGameMenu(optRace, optReady)
     if (state == 3) then -- ccs_mapinfo
       -- got ICMMap => load map
       RunJoiningMapMenu(optRace, optReady)
-      menu:stop(0)
+      if (optExtraLabel ~= nil) then
+	 menu:stop(1) -- joining through metaserver menu
+      else
+	 menu:stop(0) -- joining through local server menu
+      end
     elseif (state == 4) then -- ccs_badmap
       ErrorMenu(_("Map not available"))
       menu:stop(1)
     elseif (state == 10) then -- ccs_unreachable
-      ErrorMenu(_("Cannot reach server"))
-      menu:stop(1)
+      if optStopDirect then
+        menu:stop(1)
+      else
+        ErrorMenu(_("Cannot reach server"))
+        menu:stop(1)
+      end
     elseif (state == 12) then -- ccs_nofreeslots
       ErrorMenu(_("Server is full"))
       menu:stop(1)
@@ -308,7 +383,7 @@ function RunJoiningGameMenu(optRace, optReady)
   local listener = LuaActionListener(checkconnection)
   menu:addLogicCallback(listener)
 
-  menu:addHalfButton("Cancel (~<Esc~>)", "escape", 92, 80,
+  menu:addHalfButton(_("Cancel (~<Esc~>)"), "escape", 92, 80,
     function() menu:stop(1) end)
 
   return menu:run()
@@ -360,6 +435,130 @@ function RunJoinIpMenu()
   menu:run()
 end
 
+function RunJoiningMetaServerMenu()
+   local menu = WarMenu(nil, panel(4), false)
+   menu:setSize(288, 128)
+   menu:setPosition((Video.Width - 288) / 2, (Video.Height - 128) / 2)
+   menu:setDrawMenusUnder(true)
+
+   menu:addLabel(_("Enter meta server address:"), 144, 11)
+   local server = menu:addTextInputField(wc2.preferences.MetaServer..":"..tostring(wc2.preferences.MetaPort), 40, 38, 212)
+   local conn_label = menu:addLabel(_("Connecting..."), 40, 60, Fonts["game"], false)
+   conn_label:setVisible(false)
+
+   local ok_button = menu:addHalfButton("~!OK", "o", 24, 80, function(s) end)
+   local cancel_button = menu:addHalfButton(_("~!Cancel"), "c", 154, 80, function() menu:stop() end)
+   ok_button:setActionCallback(
+      function(s)
+	 if string.len(server:getText()) > 2 then -- len 2 means :0 -- i.e., no port, no host
+	    conn_label:setVisible(true)
+	    conn_label:setCaption(_("Connecting..."))
+	    conn_label:adjustSize()
+	    local ip = string.find(server:getText(), ":")
+	    if ip ~= nil then
+	       wc2.preferences.MetaServer = string.sub(server:getText(), 1, ip - 1)
+	       wc2.preferences.MetaPort = tonumber(string.sub(server:getText(), ip + 1))
+	       if (wc2.preferences.MetaPort == 0) then wc2.preferences.MetaPort = 7775 end
+	    else
+	       wc2.preferences.MetaServer = string.sub(server:getText(), 1)
+	       wc2.preferences.MetaPort = 7775
+	    end
+	    ok_button:setEnabled(false)
+	    cancel_button:setEnabled(false)
+	    MetaClient:SetMetaServer(wc2.preferences.MetaServer, wc2.preferences.MetaPort)
+	    if (MetaClient:Init() == -1) then
+	       conn_label:setCaption(_("Unable to connect"))
+	       conn_label:adjustSize()
+	       ok_button:setEnabled(true)
+	       cancel_button:setEnabled(true)
+	    else
+	       -- connected
+	       RunJoinOnlineMenu()
+	       menu:stop(0)
+	    end
+
+	 end
+      end
+   )
+
+   menu:run()
+end
+
+function RunJoinOnlineMenu()
+  local menu = WarMenu(nil, panel(5), false)
+  local serverlist
+  menu:setSize(352, 352)
+  menu:setPosition((Video.Width - 352) / 2, (Video.Height - 352) / 2)
+  menu:setDrawMenusUnder(true)
+  menu:addLabel(_("Open Games: "), 176, 20)
+  local servers = {}
+
+  local function ServerListUpdate()
+     serverlist = nil
+     servers = {}
+     local idx = 1
+     MetaClient:Send("LISTGAMES")
+     if (MetaClient:Recv() ~= -1) then
+	local log = MetaClient:GetLastMessage().entry
+	if string.find(log, "LISTGAMES_OK") then
+	   -- we're done
+	elseif string.find(log, "LISTGAMES") then
+	   servers[idx] = log
+	else
+	   -- what?
+	end
+     end
+     serverlist = menu:addImageListBox(20, 50, 300, 120, servers)
+  end
+  ServerListUpdate()
+  menu:addFullButton(_("Co~!nnect"), "n", 60, 180, function()
+      local stringlist = {}
+      local idx = 1
+      for i in string.gmatch(servers[serverlist:getSelected() + 1], "%S+") do
+	 stringlist[idx] = i
+	 idx = idx + 1
+      end
+      local id = stringlist[2]
+      local iplocal = stringlist[table.getn(stringlist) - 1]
+      local portlocal = tonumber(stringlist[table.getn(stringlist)])
+      MetaClient:Send("JOINGAME " .. tostring(id))
+      local ip = "0.0.0.0"
+      local port = 0
+      while (MetaClient:Recv() ~= -1) do
+	 local log = MetaClient:GetLastMessage().entry
+	 stringlist = {}
+	 if string.find(log, "JOINGAME_OK") then
+	    local i
+	    idx = 1
+	    for i in string.gmatch(log, "%S+") do
+	       stringlist[idx] = i
+	       idx = idx + 1
+	    end
+	    ip = stringlist[table.getn(stringlist) - 1]
+	    port = tonumber(stringlist[table.getn(stringlist)])
+	    NetworkSetupServerAddress(iplocal, portlocal)
+	    NetworkInitClientConnect()
+	    -- try the global ip first, so any clients outside the network may be able to do p2p later on
+	    if (RunJoiningGameMenu(nil, nil, _("Connecting via " .. tostring(iplocal) .. " (first try)"), true) ~= 0) then
+	       -- cannot connect to server-global ip address, try the local one
+	       NetworkSetupServerAddress(ip, port)
+	       NetworkInitClientConnect()
+	       if (RunJoiningGameMenu(nil, nil, _("Connecting via " .. tostring(ip) .. " (second try)"), false) ~= 0) then
+		  -- connect failed, don't leave this menu
+		  return
+	       end
+	    end
+	 end
+      end
+      return
+    end)
+  menu:addFullButton(_("~!Cancel"), "c", 60, 300, function()
+      MetaClient:Close()
+      menu:stop()
+  end)
+  menu:run()
+end
+
 function RunEditServerMenu(number)
   local menu = WarMenu(nil, panel(2), false)
   menu:setSize(288, 256)
@@ -422,8 +621,13 @@ function RunServerMultiGameMenu(map, description, numplayers, options)
   local optRace = options.race
   local optResources = options.resources
   local optUnits = options.units
+  local optAiPlayerNum = options.aiPlayerNum or 0
   local optAutostartNum = options.autostartNum
   local optDedicated = options.dedicated
+  local optOnline = options.online
+  if optOnline == nil then
+     optOnline = true
+  end
 
   menu = WarMenu(_("Create MultiPlayer game"))
 
@@ -433,7 +637,7 @@ function RunServerMultiGameMenu(map, description, numplayers, options)
   menu:writeText(_("Players:"), sx, sy*3+50)
   local players = menu:writeText(numplayers, sx+70, sy*3+50)
   menu:writeText(_("Description:"), sx, sy*3+70)
-  local descr = menu:writeText("Unknown map", sx+20, sy*3+90)
+  local descr = menu:writeText(_("Unknown map"), sx+20, sy*3+90)
 
   local function fowCb(dd)
     ServerSetupState.FogOfWar = bool2int(dd:isMarked())
@@ -482,26 +686,85 @@ function RunServerMultiGameMenu(map, description, numplayers, options)
 
   menu:writeText(_("Dedicated AI Server:"), sx, sy*12+75)
   local dedicatedCb = function (dd)
-    ServerSetupState.CompOpt[0] = bool2int(dd:isMarked())
-    LocalSetupState.CompOpt[0] = bool2int(dd:isMarked())
+    if dd:isMarked() then
+      -- 2 == closed
+      ServerSetupState.CompOpt[0] = 2
+    else
+      -- 0 == available
+      ServerSetupState.CompOpt[0] = 0
+    end
+    NetworkServerResyncClients()
   end
   local dedicated = menu:addImageCheckBox("", sx + 200, sy*12+75, offi, offi2, oni, oni2, dedicatedCb)
+  local onlineLabelTxt = _("Meta-Server")
+  local onlineLabel = menu:writeText(onlineLabelTxt, sx, sy*13+75)
+  local server = menu:addTextInputField(
+     wc2.preferences.MetaServer..":"..tostring(wc2.preferences.MetaPort), sx, sy*14+75, 200)
+  local serverCb = function (dd)
+     if string.len(server:getText()) > 2 then -- len 2 means :0 -- i.e., no port, no host
+	local ip = string.find(server:getText(), ":")
+	if ip ~= nil then
+	   wc2.preferences.MetaServer = string.sub(server:getText(), 1, ip - 1)
+	   wc2.preferences.MetaPort = tonumber(string.sub(server:getText(), ip + 1))
+	   if (wc2.preferences.MetaPort == 0) then wc2.preferences.MetaPort = 7775 end
+	else
+	   wc2.preferences.MetaServer = string.sub(server:getText(), 1)
+	   wc2.preferences.MetaPort = 7775
+	end
+	SavePreferences()
+	MetaClient:SetMetaServer(wc2.preferences.MetaServer, wc2.preferences.MetaPort)
+     end
+  end
+  serverCb(server)
+  local onlineCb = function (dd)
+     serverCb(server)
+     if (dd:isMarked()) then
+	server:setEnabled(false)
+	if (MetaClient:Init() == -1) then
+	   onlineLabel:setCaption(_("(metaserver down)"))
+	   onlineLabel:adjustSize()
+	   dd:setMarked(false)
+	   server:setEnabled(true)
+	else
+	   if (MetaClient:CreateGame(description .. ":" .. wc2.preferences.PlayerName, map, "" .. numplayers) == -1) then
+	      onlineLabel:setCaption(_("(metaserver error)"))
+	      onlineLabel:adjustSize()
+	      dd:setMarked(false)
+	      server:setEnabled(true)
+	      MetaClient:Close()
+	   else
+	      onlineLabel:setCaption(onlineLabelTxt)
+	      onlineLabel:adjustSize()
+	   end
+	end
+     else
+	server:setEnabled(true)
+	MetaClient:Send("CANCELGAME")
+	MetaClient:Close()
+     end
+  end
+  local online = menu:addImageCheckBox("", sx + 200, sy*13+75, offi, offi2, oni, oni2, onlineCb)
 
-  local updatePlayers = addPlayersList(menu, numplayers)
+  GameSettings.Opponents = optAiPlayerNum
+  local updatePlayers = addPlayersList(menu, numplayers, true)
 
   NetworkMapName = map
   NetworkInitServerConnect(numplayers)
   ServerSetupState.FogOfWar = 1
-  ServerSetupState.Opponents = 15
-  GameSettings.Opponents = 15
+  ServerSetupState.Opponents = optAiPlayerNum
   local function startFunc(s)
     SetFogOfWar(fow:isMarked())
     if revealmap:isMarked() == true then
       RevealMap()
     end
     NetworkServerStartGame()
+    print("Metaserver stargame...")
+    MetaClient:Send("STARTGAME") -- only the host's message will actually be processed
+    print("...metaserver game started")
     NetworkGamePrepareGameSettings()
     RunMap(map)
+    MetaClient:Send("ENDGAME finished") -- only the host's message will actually be processed
+    MetaClient:Close()
     menu:stop()
   end
   local startgame = menu:addFullButton(_("~!Start Game"), "s", sx * 11, sy*14, startFunc)
@@ -522,7 +785,10 @@ function RunServerMultiGameMenu(map, description, numplayers, options)
         dedicated:setMarked(true)
         dedicatedCb(dedicated)
         optDedicated = false
-
+      elseif optOnline then
+        online:setMarked(true)
+        onlineCb(online)
+        optOnline = false
       elseif (optRace == "human" or optRace == "Human") then
         race:setSelected(1)
         raceCb(race)
@@ -583,7 +849,17 @@ function RunServerMultiGameMenu(map, description, numplayers, options)
   updateStartButton(updatePlayers())
 
   menu:addFullButton(_("~!Cancel"), "c", Video.Width / 2 - 100, Video.Height - 100,
-    function() InitGameSettings(); menu:stop() end)
+		     function()
+			InitGameSettings()
+			-- the MetaClient message will not be processed if this
+			-- game is not registered. For the host, the server will
+			-- also ensure the entire game is marked as cancelled.
+			print("Leaving metaserver...")
+			MetaClient:Send("PARTGAME")
+			MetaClient:Close()
+			print("...left")
+			menu:stop()
+  end)
 
   menu:run()
 end
@@ -681,7 +957,7 @@ function RunMultiPlayerGameMenu(s)
   menu:writeText(_("Nickname :"), 208 + offx, 248 + offy)
   nick = menu:addTextInputField(GetLocalPlayerName(), offx + 298, 244 + offy)
 
-  menu:addFullButton(_("~!Join Game"), "j", 208 + offx, 298 + (36 * 0) + offy,
+  menu:addFullButton(_("~!Join Local Game"), "j", 208 + offx, 298 + (36 * 0) + offy,
     function()
       if nick:getText() ~= GetLocalPlayerName() then
         SetLocalPlayerName(nick:getText())
@@ -691,7 +967,17 @@ function RunMultiPlayerGameMenu(s)
       RunJoinIpMenu()
       FixMusic()
     end)
-  menu:addFullButton(_("~!Create Game"), "c", 208 + offx, 298 + (36 * 1) + offy,
+  menu:addFullButton(_("Join ~!Online Game"), "o", 208 + offx, 298 + (36 * 1) + offy,
+    function()
+      if nick:getText() ~= GetLocalPlayerName() then
+        SetLocalPlayerName(nick:getText())
+        wc2.preferences.PlayerName = nick:getText()
+        SavePreferences()
+      end
+      RunJoiningMetaServerMenu()
+      FixMusic()
+    end)
+  menu:addFullButton(_("~!Create Game"), "c", 208 + offx, 298 + (36 * 2) + offy,
     function()
       if nick:getText() ~= GetLocalPlayerName() then
         SetLocalPlayerName(nick:getText())
@@ -701,15 +987,6 @@ function RunMultiPlayerGameMenu(s)
       RunCreateMultiGameMenu()
       FixMusic()
     end)
-  -- menu:addFullButton(_("Meta~!server"), "s", 208 + offx, 298 + (36 * 2) + offy,
-  --   function()
-  --     if nick:getText() ~= GetLocalPlayerName() then
-  --       SetLocalPlayerName(nick:getText())
-  --       wc2.preferences.PlayerName = nick:getText()
-  --       SavePreferences()
-  --     end
-  --     RunJoiningMetaServerMenu()
-  --   end)
 
   menu:addFullButton(_("~!Previous Menu"), "p", 208 + offx, 298 + (36 * 3) + offy,
     function() menu:stop() end)
