@@ -167,6 +167,7 @@ DefineUnitType("unit-catapult", { Name = _("Catapult"),
   RightMouseAction = "attack",
   CanAttack = true,
   CanTargetLand = true, CanTargetSea = true,
+  GroundAttack = true,
   SelectableByRectangle = true,
   Sounds = {
     "selected", "catapult-selected",
@@ -195,6 +196,7 @@ DefineUnitType("unit-catapult-super", { Name = _("Catapult"),
   RightMouseAction = "attack",
   CanAttack = true,
   CanTargetLand = true, CanTargetSea = true, CanTargetAir = true,
+  GroundAttack = true,
   SelectableByRectangle = false,
   Sounds = {
     "selected", "catapult-selected",
@@ -602,6 +604,7 @@ DefineUnitType("unit-ogre-juggernaught", { Name = _("Ogre Juggernaught"),
   RightMouseAction = "attack",
   CanAttack = true,
   CanTargetLand = true, CanTargetSea = true,
+  GroundAttack = true,
   SeaUnit = true,
   SideAttack = true, 
   SelectableByRectangle = true,
@@ -725,6 +728,7 @@ DefineUnitType("unit-zeppelin", { Name = _("Goblin Zeppelin"),
   AirUnit = true, Coward = true,
   DetectCloak = true,
   SelectableByRectangle = true,
+  OnReady = AiExploreUnit,
   Sounds = {
     "selected", "goblin-zeppelin-selected",
     "acknowledge", "goblin-zeppelin-acknowledge",
@@ -744,7 +748,7 @@ DefineUnitType("unit-eye-of-vision", { Name = _("Eye of Kilrogg"),
   MaxAttackRange = 1,
   Priority = 0,
   DecayRate = 3,
-  Demand = 1,
+  Demand = 0,
   Type = "fly", ShadowFly = {Value = 2, Enable = true},
   RightMouseAction = "move",
   AirUnit = true,
