@@ -30,6 +30,26 @@
 -- For documentation see stratagus/doc/scripts/scripts.html
 DebugPrint("Stratagus default config file loading ...\n")
 
+if (_VERSION == "Lua 5.2" or _VERSION == "Lua 5.3") then
+
+  function table:getn()
+    return #self
+  end
+
+  function table:foreach(f)
+    for key,value in pairs(self) do
+      f(key, value)
+    end
+  end
+
+  function table:foreachi(f)
+    for k, v in ipairs (self) do
+      f(k, v)
+    end 
+  end
+
+end
+
 -------------------------------------------------------------------------------
 --  Version
 -------------------------------------------------------------------------------
@@ -40,16 +60,14 @@ Load("scripts/wc2-config.lua")
 
 if (wargus.tales == true) then
 	wargus.Name = _("Aleona's Tales")
-	wargus.Homepage = "stratagus.com"
-	wargus.Copyright = _("(c) 2012-2015 by The Stratagus Project.")
 else
 	wargus.Name = _("Wargus")
-	wargus.Homepage = "wargus.stratagus.com"
-	wargus.Copyright = _("(c) 1998-2015 by The Stratagus Project.")
 end
+wargus.Homepage = "https://wargus.github.io"
+wargus.Copyright = _("(c) 1998-2020 by The Stratagus Project.")
 
-wargus.Version = "2.4.1"
-wargus.Licence = "GPL v2"
+wargus.Version = "2.4.3"
+wargus.Licence = "GPL v2+"
 
 
 -------------------------------------------------------------------------------
@@ -101,6 +119,7 @@ DefineBoolFlags("Center", "WoodImprove", "OilImprove")
 
 --  Enter your default title screen.
 SetTitleScreens(
+  {Image = "ui/black_title.png", Timeout = 1},
   {Image = "videos/logo.ogv"},
   {Image = "videos/gameintro.ogv"},
   {Image = "ui/title.png", Music = "music/Orc Briefing" .. wargus.music_extension, Timeout = 20}
@@ -394,12 +413,14 @@ local defaultPreferences = {
 	UseFancyBuildings = false,      --  Enable/disable fancy building (random mirroring buildings)
 	UseOpenGL = false,
 	VideoFullScreen = false,
-	VideoHeight = 600,
-	VideoWidth = 800,
+	VideoViewportHeight = 600,
+	VideoViewportWidth = 800,
+	VideoShaderIndex = 0,
+	ZoomNoResize = false,
 	ViewportMode = 0,
 	ServerList = {},
-	MetaServer = "localhost",
-	MetaPort = 7775,
+	MetaServer = "",
+	MetaPort = 0,
 	SF2SoundFont = "music/TimGM6mb.sf2"
 }
 
@@ -465,7 +486,14 @@ SetMusicVolume(wc2.preferences.MusicVolume)
 SetTranslationsFiles(wc2.preferences.StratagusTranslation, wc2.preferences.GameTranslation)
 SetUseOpenGL(wc2.preferences.UseOpenGL)
 SetVideoFullScreen(wc2.preferences.VideoFullScreen)
-SetVideoResolution(wc2.preferences.VideoWidth, wc2.preferences.VideoHeight)
+SetVideoResolution(wc2.preferences.VideoViewportWidth, wc2.preferences.VideoViewportHeight)
+if (wc2.preferences.ZoomNoResize) then
+   SetZoomNoResize(640, 480) -- the original Warcraft was 640x480
+else
+   SetZoomNoResize(false) -- disable
+end
+Video.ShaderIndex = wc2.preferences.VideoShaderIndex
+SwitchToShader()
 
 UI.ButtonPanel.ShowCommandKey = wc2.preferences.ShowCommandKey
 Preference.SF2Soundfont = wc2.preferences.SF2SoundFont
