@@ -10,7 +10,7 @@
 //
 /**@name wartool.c - Extract files from war archives. */
 //
-//      (c) Copyright 1999-2015 by Lutz Sammer, Nehal Mistry, Jimmy Salmon,
+//      (c) Copyright 1999-2016 by Lutz Sammer, Nehal Mistry, Jimmy Salmon,
 //                                 Pali Rohár and cybermind
 //
 //      This program is free software; you can redistribute it and/or modify
@@ -35,6 +35,7 @@
 ----------------------------------------------------------------------------*/
 
 #include "wartool.h"
+#include <stratagus-gameutils.h>
 
 #if defined(_MSC_VER) || defined(WIN32)
 #include <windows.h>
@@ -312,12 +313,10 @@ int OpenArchive(const char* file, int type)
 	//
 	f = open(file, O_RDONLY | O_BINARY, 0);
 	if (f == -1) {
-		printf("Can't open %s\n", file);
-		exit(-1);
+		error("Can't open file", file);
 	}
 	if (stat(file, &stat_buf)) {
-		printf("Can't stat %s\n", file);
-		exit(-1);
+		error("Can't stat file", file);
 	}
 //	printf("Filesize %ld %ldk\n",
 //		(long)stat_buf.st_size, stat_buf.st_size / 1024);
@@ -328,11 +327,11 @@ int OpenArchive(const char* file, int type)
 	buf = (unsigned char *)malloc(stat_buf.st_size);
 	if (!buf) {
 		printf("Can't malloc %ld\n", (long)stat_buf.st_size);
-		exit(-1);
+		error("Memory error", "Could not allocate enough memory to read archive.");
 	}
 	if (read(f, buf, stat_buf.st_size) != stat_buf.st_size) {
 		printf("Can't read %ld\n", (long)stat_buf.st_size);
-		exit(-1);
+		error("Memory error", "Could not read archive into RAM.");
 	}
 	close(f);
 
@@ -341,7 +340,7 @@ int OpenArchive(const char* file, int type)
 //	printf("Magic\t%08X\t", i);
 	if (i != 0x19) {
 		printf("Wrong magic %08x, expected %08x\n", i, 0x00000019);
-		exit(-1);
+		error("Archive version error", "This version of the data is not supported");
 	}
 	entries = FetchLE16(cp);
 //	printf("Entries\t%5d\t", entries);
@@ -349,7 +348,7 @@ int OpenArchive(const char* file, int type)
 //	printf("ID\t%d\n", i);
 	if (i != type) {
 		printf("Wrong type %08x, expected %08x\n", i, type);
-		exit(-1);
+		error("Archive version error", "This version of the data is not supported");
 	}
 
 	//
@@ -358,7 +357,7 @@ int OpenArchive(const char* file, int type)
 	op = (unsigned char **)malloc((entries + 1) * sizeof(unsigned char*));
 	if (!op) {
 		printf("Can't malloc %d entries\n", entries);
-		exit(-1);
+		error("Memory error", "Could not allocate enough memory to read archive.");
 	}
 	for (i = 0; i < entries; ++i) {
 		op[i] = buf + FetchLE32(cp);
@@ -411,11 +410,11 @@ unsigned char* ExtractEntry(unsigned char* cp, size_t* lenp)
 	dp = dest = (unsigned char *)malloc(uncompressed_length);
 	if (!dest) {
 		printf("Can't malloc %d\n", (int)uncompressed_length);
-		exit(-1);
+		error("Memory error", "Could not allocate enough memory to read archive.");
 	}
 
 	if (flags == 0x20) {
-		unsigned char buf[4096];
+		unsigned char buf[8192] = {'\0'};
 		unsigned char* ep;
 		int bi;
 
@@ -466,7 +465,7 @@ unsigned char* ExtractEntry(unsigned char* cp, size_t* lenp)
 		memcpy(dest, cp, uncompressed_length);
 	} else {
 		printf("Unknown flags %x\n", flags);
-		exit(-1);
+		error("Archive version error", "This version of the data is not supported");
 	}
 
 	// return resulting length
@@ -583,7 +582,7 @@ unsigned char* ConvertPalette(unsigned char* pal)
 int ConvertRgb(const char* file, int rgbe)
 {
 	unsigned char* rgbp;
-	char buf[1024];
+	char buf[8192] = {'\0'};
 	FILE* f;
 	int i;
 	size_t l;
@@ -600,7 +599,7 @@ int ConvertRgb(const char* file, int rgbe)
 	if (!f) {
 		perror("");
 		printf("Can't open %s\n", buf);
-		exit(-1);
+		error("Memory error", "Could not allocate enough memory to read archive.");
 	}
 	if (l != fwrite(rgbp, 1, l, f)) {
 		printf("Can't write %d bytes\n", (int)l);
@@ -618,7 +617,7 @@ int ConvertRgb(const char* file, int rgbe)
 	if (!f) {
 		perror("");
 		printf("Can't open %s\n", buf);
-		exit(-1);
+		error("Memory error", "Could not allocate enough memory to read archive.");
 	}
 	fprintf(f, "GIMP Palette\n# Stratagus %c%s -- GIMP Palette file\n",
 		toupper(*file), file + 1);
@@ -716,8 +715,8 @@ static void SaveCCL(char* name, unsigned char* map __attribute__((unused)),
 	int i;
 	char* cp;
 	FILE* f;
-	char file[1024];
-	char tileset[1024];
+	char file[8192] = {'\0'};
+	char tileset[8192] = {'\0'};
 
 	f = stdout;
 	// FIXME: open file!
@@ -839,7 +838,7 @@ int ConvertTileset(const char* file, int pale, int mege, int mine, int mape)
 	int w;
 	int h;
 	size_t megl;
-	char buf[1024];
+	char buf[8192] = {'\0'};
 
 	palp = ExtractEntry(ArchiveOffsets[pale], NULL);
 	megp = ExtractEntry(ArchiveOffsets[mege], &megl);
@@ -1087,7 +1086,7 @@ unsigned char* ConvertGraphic(int gfx, unsigned char* bp, int *wp, int *hp,
 	//         5, 6, 7, 8, 9, ...
 	if (!image) {
 		printf("Can't allocate image\n");
-		exit(-1);
+		error("Memory error", "Could not allocate enough memory to read archive.");
 	}
 	// Set all to transparent.
 	memset(image, 255, best_width * best_height * length);
@@ -1169,7 +1168,7 @@ int ConvertGfu(const char* file,int pale,int gfue)
 	unsigned char* image;
 	int w;
 	int h;
-	char buf[1024];
+	char buf[8192] = {'\0'};
 
 	palp = ExtractEntry(ArchiveOffsets[pale], NULL);
 	gfup = ExtractEntry(ArchiveOffsets[gfue], NULL);
@@ -1199,7 +1198,7 @@ int ConvertGroupedGfu(const char *path, int pale, int gfue, int glist)
 	unsigned char* image;
 	int w;
 	int h;
-	char buf[1024];
+	char buf[8192] = {'\0'};
 	int i;
 	const GroupedGraphic *gg;
 
@@ -1247,7 +1246,7 @@ int ConvertGroupedGfu(const char *path, int pale, int gfue, int glist)
 void ConvertPud(const char* file, int pude, bool justconvert = false)
 {
 	unsigned char* pudp = NULL;
-	char buf[1024];
+	char buf[8192] = {'\0'};
 	size_t l;
 
 	if (justconvert == false) {
@@ -1260,7 +1259,7 @@ void ConvertPud(const char* file, int pude, bool justconvert = false)
 
 		PudToStratagus(pudp, l, strrchr(file, '/') + 1, buf);
 	} else {
-		char pudfile[1024];
+		char pudfile[8192] = {'\0'};
 		struct stat sb;
 		size_t filesize = 0;
 		FILE *f = NULL;
@@ -1298,18 +1297,18 @@ void ConvertPud(const char* file, int pude, bool justconvert = false)
 */
 void ConvertFilePuds(const char **pudlist)
 {
-	char pudname[1024];
-	char base[1024];
-	char outdir[1024];
+	char pudname[8192] = {'\0'};
+	char base[8192] = {'\0'};
+	char outdir[8192] = {'\0'};
 	unsigned char *puddata;
 	struct stat sb;
 	FILE *f;
 	int i;
 
 	for (i = 0; pudlist[i][0] != '\0'; ++i) {
-		char origname[1024];
+		char origname[8192] = {'\0'};
 		if (CDType & CD_UPPER) {
-			char filename[1024];
+			char filename[8192] = {'\0'};
 			int j = 0;
 			strcpy(filename, pudlist[i]);
 			strcpy(origname, pudlist[i]);
@@ -1415,7 +1414,7 @@ unsigned char* ConvertFnt(unsigned char* start, int *wp, int *hp)
 	image = (unsigned char *)malloc(image_width * image_height);
 	if (!image) {
 		printf("Can't allocate image\n");
-		exit(-1);
+		error("Memory error", "Could not allocate enough memory to read archive.");
 	}
 	memset(image, 255, image_width * image_height);
 
@@ -1519,7 +1518,7 @@ int ConvertFont(const char* file, int pale, int fnte)
 	unsigned char* image;
 	int w;
 	int h;
-	char buf[1024];
+	char buf[8192] = {'\0'};
 
 	palp = ExtractEntry(ArchiveOffsets[pale], NULL);
 	fntp = ExtractEntry(ArchiveOffsets[fnte], NULL);
@@ -1566,7 +1565,7 @@ unsigned char* ConvertImg(unsigned char* bp, int* wp, int* hp)
 	image = (unsigned char *)malloc(width * height);
 	if (!image) {
 		printf("Can't allocate image\n");
-		exit(-1);
+		error("Memory error", "Could not allocate enough memory to read archive.");
 	}
 	memcpy(image, bp, width * height);
 
@@ -1623,7 +1622,7 @@ int ConvertImage(const char* file, int pale, int imge, int nw, int nh)
 	unsigned char* image;
 	int w;
 	int h;
-	char buf[1024];
+	char buf[8192] = {'\0'};
 
 	// Workaround for MAC expansion CD
 	if (CDType & CD_MAC) {
@@ -1646,7 +1645,7 @@ int ConvertImage(const char* file, int pale, int imge, int nw, int nh)
 	if (!image) {
 		printf("Please report this bug, could not extract image: file=%s pale=%d imge=%d nw=%d nh=%d mac=%d\n",
 			file, pale, imge, nw, nh, CDType & CD_MAC);
-		exit(-1);
+		error("Archive version error", "This version of the CD is not supported");
 	}
 	free(imgp);
 	ConvertPalette(palp);
@@ -1697,7 +1696,7 @@ unsigned char* ConvertCur(unsigned char* bp, int* wp, int* hp)
 	image = (unsigned char *)malloc(width * height);
 	if (!image) {
 		printf("Can't allocate image\n");
-		exit(-1);
+		error("Memory error", "Could not allocate enough memory to read archive.");
 	}
 	for (i = 0; i < width * height; ++i) {
 		image[i] = bp[i] ? bp[i] : 255;
@@ -1719,7 +1718,7 @@ int ConvertCursor(const char* file, int pale, int cure)
 	unsigned char* image;
 	int w;
 	int h;
-	char buf[1024];
+	char buf[8192] = {'\0'};
 
 	if (pale == 27 && cure == 314 && Pal27 ) { // Credits arrow (Blue arrow NW)
 		palp = Pal27;
@@ -1755,7 +1754,7 @@ int ConvertCursor(const char* file, int pale, int cure)
 int ConvertWav(const char* file, int wave)
 {
 	unsigned char* wavp;
-	char buf[1024];
+	char buf[8192] = {'\0'};
 	gzFile gf;
 	size_t l;
 
@@ -1767,7 +1766,7 @@ int ConvertWav(const char* file, int wave)
 	if (!gf) {
 		perror("");
 		printf("Can't open %s\n", buf);
-		exit(-1);
+		error("Memory error", "Could not allocate enough memory to read archive.");
 	}
 	if (l != (size_t)gzwrite(gf, wavp, l)) {
 		printf("Can't write %d bytes\n", (int)l);
@@ -1792,7 +1791,7 @@ int ConvertXmi(const char* file, int xmi)
 {
 	unsigned char* xmip;
 	unsigned char* midp;
-	char buf[1024];
+	char buf[8192] = {'\0'};
 	FILE *f;
 	size_t xmil;
 	size_t midl;
@@ -1807,7 +1806,7 @@ int ConvertXmi(const char* file, int xmi)
 	if (!f) {
 		perror("");
 		printf("Can't open %s\n", buf);
-		exit(-1);
+		error("Memory error", "Could not allocate enough memory to read archive.");
 	}
 	if (midl != fwrite(midp, 1, midl, f)) {
 		printf("Can't write %d bytes\n", (int)midl);
@@ -1834,18 +1833,20 @@ int CopyFile(char *from, char *to, int overwrite)
 {
 	struct stat st;
 	char *cmd;
+	int cmdlen;
 	int ret;
 
 	if (!overwrite && !stat(to, &st))
 		return 0;
 
-	cmd = (char *)calloc(strlen("cp \"") + strlen(from) + strlen("\" \"") + strlen(to) + strlen("\"") + 1, 1);
+	cmdlen = strlen("cp \"") + strlen(from) + strlen("\" \"") + strlen(to) + strlen("\" ");
+	cmd = (char *)calloc(cmdlen + 1, 1);
 	if (!cmd) {
 		fprintf(stderr, "Memory error\n");
-		exit(-1);
+		error("Memory error", "Could not allocate enough memory to read archive.");
 	}
 
-	sprintf(cmd, "cp \"%s\" \"%s\"", from, to);
+	snprintf(cmd, cmdlen, "cp \"%s\" \"%s\"", from, to);
 	ret = system(cmd);
 	free(cmd);
 
@@ -1863,8 +1864,8 @@ int CopyFile(char *from, char *to, int overwrite)
 int CopyMusic(void)
 {
 	struct stat st;
-	char buf1[1024];
-	char buf2[1024];
+	char buf1[8192] = {'\0'};
+	char buf2[8192] = {'\0'};
 	char ext[4];
 	int i;
 	int count = 0;
@@ -1901,25 +1902,27 @@ int CopyMusic(void)
 int ConvertMusic(void)
 {
 	struct stat st;
-	char buf[1024];
+	char buf[8192] = {'\0'};
 	char *cmd;
+	int cmdlen;
 	int ret, i;
 	int count = 0;
 
 	for ( i = 0; MusicNames[i]; ++i ) {
-		sprintf(buf, "%s/%s/%s.wav", Dir, MUSIC_PATH, MusicNames[i]);
+		snprintf(buf, 4095, "%s/%s/%s.wav", Dir, MUSIC_PATH, MusicNames[i]);
 		CheckPath(buf);
 
 		if (stat(buf, &st))
 			continue;
 
-		cmd = (char*) calloc(strlen("ffmpeg2theora --optimize \"") + strlen(buf) + strlen("\" -o \"") + strlen(buf) + strlen("\"") + 1, 1);
+		cmdlen = strlen("ffmpeg -y -i \"") + strlen(buf) + strlen("\" \"") + strlen(buf) + strlen("\" ");
+		cmd = (char*) calloc(cmdlen + 1, 1);
 		if (!cmd) {
 			fprintf(stderr, "Memory error\n");
-			exit(-1);
+			error("Memory error", "Could not allocate enough memory to read archive.");
 		}
 
-		sprintf(cmd, "ffmpeg2theora --optimize \"%s\" -o \"%s/%s/%s.ogg\"", buf, Dir, MUSIC_PATH, MusicNames[i]);
+		snprintf(cmd, cmdlen, "ffmpeg -y -i \"%s\" \"%s/%s/%s.ogg\"", buf, Dir, MUSIC_PATH, MusicNames[i]);
 
 		ret = system(cmd);
 
@@ -1927,7 +1930,7 @@ int ConvertMusic(void)
 		remove(buf);
 
 		if (ret != 0) {
-			printf("Can't convert wav sound %s to ogv format. Is ffmpeg2theora installed in PATH?\n", MusicNames[i]);
+			printf("Can't convert wav sound %s to ogg format. Is ffmpeg installed in PATH?\n", MusicNames[i]);
 			fflush(stdout);
 		}
 
@@ -1941,13 +1944,14 @@ int ConvertMusic(void)
 			if (stat(buf, &st))
 				continue;
 
-			cmd = (char*) calloc(strlen("ffmpeg2theora --optimize \"") + strlen(buf) + strlen("\" -o \"") + strlen(buf) + strlen("\"") + 1, 1);
+			cmdlen = strlen("ffmpeg -y -i \"") + strlen(buf) + strlen("\" \"") + strlen(buf) + strlen("\" ");
+			cmd = (char*) calloc(cmdlen + 1, 1);
 			if (!cmd) {
 				fprintf(stderr, "Memory error\n");
-				exit(-1);
+				error("Memory error", "Could not allocate enough memory to read archive.");
 			}
 
-			sprintf(cmd, "ffmpeg2theora --optimize \"%s\" -o \"%s/%s/%s.ogg\"", buf, Dir, MUSIC_PATH, BNEMusicNames[i]);
+			snprintf(cmd, cmdlen, "ffmpeg -y -i \"%s\" \"%s/%s/%s.ogg\"", buf, Dir, MUSIC_PATH, BNEMusicNames[i]);
 
 			ret = system(cmd);
 
@@ -1955,7 +1959,7 @@ int ConvertMusic(void)
 			remove(buf);
 
 			if (ret != 0) {
-				printf("Can't convert wav sound %s to ogv format. Is ffmpeg2theora installed in PATH?\n", BNEMusicNames[i]);
+				printf("Can't convert wav sound %s to ogg format. Is ffmpeg installed in PATH?\n", BNEMusicNames[i]);
 				fflush(stdout);
 			}
 
@@ -1979,13 +1983,15 @@ int ConvertMusic(void)
 int ConvertVideo(const char* file, int video, bool justconvert = false)
 {
 	unsigned char* vidp;
-	char buf[1024];
+	char buf[8192] = {'\0'};
 	char* cmd;
 	FILE* f;
 	size_t l;
 	int ret;
+	int cmdlen;
+	char outputfile[8192] = {'\0'};
 
-	sprintf(buf,"%s/%s.smk", Dir, file);
+	snprintf(buf,4095,"%s/%s.smk", Dir, file);
 	CheckPath(buf);
 	if (justconvert == false) {
 		vidp = ExtractEntry(ArchiveOffsets[video], &l);
@@ -1994,7 +2000,7 @@ int ConvertVideo(const char* file, int video, bool justconvert = false)
 		if (!f) {
 			perror("");
 			printf("Can't open %s\n", buf);
-			exit(-1);
+			error("Memory error", "Could not allocate enough memory to read archive.");
 		}
 		if (l != fwrite(vidp, 1, l, f)) {
 			printf("Can't write %d bytes\n", (int)l);
@@ -2005,21 +2011,24 @@ int ConvertVideo(const char* file, int video, bool justconvert = false)
 		fclose(f);
 	}
 
-	cmd = (char*) calloc(strlen("ffmpeg2theora --optimize \"") + strlen(buf) + strlen("\" -o \"") + strlen(buf) + strlen("\"") + 1, 1);
+	cmdlen = strlen("ffmpeg -y -i \"") + strlen(buf) + strlen("\" -codec:v libtheora -qscale:v 31 -codec:a libvorbis -qscale:a 15 -pix_fmt yuv420p -aspect 4:3 -vf scale=640x0,setsar=1:1 \"") + strlen(buf) + strlen("\" ");
+	cmd = (char*) calloc(cmdlen + 1, 1);
 	if (!cmd) {
 		fprintf(stderr, "Memory error\n");
-		exit(-1);
+		error("Memory error", "Could not allocate enough memory to read archive.");
 	}
 
-	sprintf(cmd, "ffmpeg2theora --optimize \"%s/%s.smk\" -o \"%s/%s.ogv\"", Dir, file, Dir, file);
-
+	snprintf(cmd, cmdlen, "ffmpeg -y -i \"%s/%s.smk\" -codec:v libtheora -qscale:v 31 -codec:a libvorbis -qscale:a 15 -pix_fmt yuv420p -aspect 4:3 -vf scale=640:0,setsar=1:1 \"%s/%s.ogv\"", Dir, file, Dir, file);
+	printf("%s\n", cmd);
 	ret = system(cmd);
 
 	free(cmd);
 	remove(buf);
 
 	if (ret != 0) {
-		printf("Can't convert video %s to ogv format. Is ffmpeg2theora installed in PATH?\n", file);
+		sprintf(outputfile, "%s/%s.ogv", Dir, file);
+		unlink(outputfile);
+		printf("Can't convert video %s to ogv format. Is ffmpeg installed in PATH?\n", file);
 		fflush(stdout);
 		return ret;
 	}
@@ -2077,7 +2086,7 @@ unsigned char *ConvertString(unsigned char *buf, size_t len)
 int ConvertText(const char* file, int txte, int ofs)
 {
 	unsigned char* txtp;
-	char buf[1024];
+	char buf[8192] = {'\0'};
 	gzFile gf;
 	size_t l;
 	size_t l2;
@@ -2101,7 +2110,7 @@ int ConvertText(const char* file, int txte, int ofs)
 	if (!gf) {
 		perror("");
 		printf("Can't open %s\n", buf);
-		exit(-1);
+		error("Memory error", "Could not allocate enough memory to read archive.");
 	}
 	str = ConvertString(txtp + ofs, l - ofs);
 	l2 = strlen((char *)str) + 1;
@@ -2155,7 +2164,7 @@ int SetupNames(const char* file __attribute__((unused)), int txte __attribute__(
 */
 char* ParseString(const char* input)
 {
-	static char buf[1024];
+	static char buf[8192] = {'\0'};
 	const char* sp;
 	char* strsp;
 	char* dp;
@@ -2209,7 +2218,7 @@ char* ParseString(const char* input)
 int CampaignsCreate(const char* file __attribute__((unused)), int txte, int ofs)
 {
 	unsigned char* objectives;
-	char buf[1024];
+	char buf[8192] = {'\0'};
 	unsigned char* CampaignData[2][26][10];
 	unsigned char* current;
 	unsigned char* next;
@@ -2246,12 +2255,12 @@ int CampaignsCreate(const char* file __attribute__((unused)), int txte, int ofs)
 	objectives = ExtractEntry(ArchiveOffsets[txte], &l);
 	if (!objectives) {
 		printf("Objectives allocation failed\n");
-		exit(-1);
+		error("Memory error", "Could not allocate enough memory to read archive.");
 	}
 	objectives = (unsigned char *)realloc(objectives, l + 1);
 	if (!objectives) {
 		printf("Objectives allocation failed\n");
-		exit(-1);
+		error("Memory error", "Could not allocate enough memory to read archive.");
 	}
 	objectives[l] = '\0';
 
@@ -2418,13 +2427,13 @@ destination-directory\tDirectory where the extracted files are placed.\n"
 int main(int argc, char** argv)
 {
 	unsigned u;
-	char buf[1024];
+	char buf[8192] = {'\0'};
 	struct stat st;
 	int expansion_cd = 0;
 	int video = 0;
 	int rip = 0;
 	int a = 1;
-	char filename[1024];
+	char filename[8192] = {'\0'};
 	FILE* f;
 
 	while (argc >= 2) {
@@ -2489,12 +2498,22 @@ int main(int argc, char** argv)
 	}
 
 	// Detect if CD is Mac/Dos, Expansion/Original/BNE, and language
-	sprintf(buf, "%s/SUPPORT/TOMES/TOME.1", ArchiveDir);
-	sprintf(filename, "%s/SUPPORT/TOMES/TOME.4", ArchiveDir);
+	sprintf(buf, "%s/support/tomes/tome.1", ArchiveDir);
 	if (!stat(buf, &st)) {
+		sprintf(filename, "%s/support/tomes/tome.4", ArchiveDir);
 		printf("Detected BNE CD\n");
 		fflush(stdout);
 		CDType |= CD_BNE | CD_EXPANSION | CD_US;
+	} else if (sprintf(buf, "%s/Support/TOMES/TOME.1", ArchiveDir) && !stat(buf, &st)) {
+		sprintf(filename, "%s/Support/TOMES/TOME.4", ArchiveDir);
+		printf("Detected BNE CD Captialized\n");
+		fflush(stdout);
+		CDType |= CD_BNE | CD_BNE_CAPS | CD_EXPANSION | CD_US;
+	} else if (sprintf(buf, "%s/SUPPORT/TOMES/TOME.1", ArchiveDir) && !stat(buf, &st)) {
+		sprintf(filename, "%s/SUPPORT/TOMES/TOME.4", ArchiveDir);
+		printf("Detected BNE CD Uppercase\n");
+		fflush(stdout);
+		CDType |= CD_BNE | CD_BNE_UPPER | CD_EXPANSION | CD_US;
 	} else {
 		sprintf(buf, "%s/rezdat.war", ArchiveDir);
 		sprintf(filename, "%s/strdat.war", ArchiveDir);
@@ -2508,7 +2527,9 @@ int main(int argc, char** argv)
 			sprintf(buf, "%s/War Resources", ArchiveDir);
 			if (stat(buf, &st)) {
 				printf("Could not find Warcraft 2 Data\n");
-				exit(-1);
+				error("Data not found", "Could not find Warcraft 2 data in folder. "
+					  "Make sure you have selected the DATA directory of the DOS version, "
+					  "or the root of the Battle.net CD.");
 			}
 			if (expansion_cd == -1 || (expansion_cd != 1 && st.st_size != 2876978)) {
 				printf("Detected original MAC CD\n");
@@ -2597,7 +2618,8 @@ int main(int argc, char** argv)
 	if (CDType & CD_BNE) {
 #ifndef USE_STORMLIB
 		printf("Please compile wartool with StormLib library to extract the data.\n");
-		exit(-1);
+		error("Archive version error", "The Battle.net edition is not supported by this version of "
+			  "the extractor tool. Please compile wartool with StormLib.");
 #endif
 	}
 
@@ -2637,6 +2659,16 @@ int main(int argc, char** argv)
 					for (int i = 0; i < sizeof(BNEReplaceTable) / sizeof(*BNEReplaceTable) ; i += 2) {
 						if (!strcmp(BNEReplaceTable[i], Todo[u].File)) {
 							Todo[u].File = BNEReplaceTable[i + 1];
+							if (CDType & CD_BNE_CAPS) {
+								Todo[u].File = BNEReplaceTableCaps[i + 1];
+							} else if (CDType & CD_BNE_UPPER) {
+								strcpy(filename, Todo[u].File);
+								while (filename[i]) {
+									filename[i] = toupper(filename[i]);
+									++i;
+								}
+								Todo[u].File = filename;
+							}
 							break;
 						}
 					}
@@ -2660,17 +2692,69 @@ int main(int argc, char** argv)
 			case Q:
 				if (!(CDType & CD_BNE)) {
 					printf("Error - not a BNE disk\n");
-					exit(-1);
+					error("Archive version error", "This version of the CD is not supported");
 				}
 #ifdef USE_STORMLIB
-				char mpqfile[256];
-				char extract[256];
+				char mpqfile[8192];
+				char extract[8192];
+				memset(mpqfile, 0, 8192);
+				memset(extract, 0, 8192);
 				if (Todo[u].Arg1 == 1) { // local archive
 					sprintf(mpqfile, "%s/%s", Dir, Todo[u].MPQFile);
 					printf("%s from MPQ file \"%s\"\n", Todo[u].ArcFile, mpqfile);
 				} else {
-					sprintf(mpqfile, "%s/%s", ArchiveDir, Todo[u].MPQFile);
-					printf("MPQ file \"%s\"\n", mpqfile);
+					char* filename = strdup(Todo[u].MPQFile);
+					// initially all lowercase
+					for (int i = 0; i < strlen(filename); i++) {
+						filename[i] = tolower(filename[i]);
+					}
+					// let's see....
+					sprintf(mpqfile, "%s/%s", ArchiveDir, filename);
+					if (stat(mpqfile, &st)) {
+						// try with initial uppercase
+						filename[0] = toupper(filename[0]);
+						sprintf(mpqfile, "%s/%s", ArchiveDir, filename);
+					}
+					if (stat(mpqfile, &st)) {
+						// try all uppercase
+						for (int i = 0; i < strlen(filename); i++) {
+							filename[i] = toupper(filename[i]);
+						}
+						sprintf(mpqfile, "%s/%s", ArchiveDir, filename);
+					}
+					if (stat(mpqfile, &st)) {
+						// try with alternative extension (mpq/exe)
+						free(filename);
+						filename = strdup(Todo[u].MPQFile);
+						// initially all lowercase
+						for (int i = 0; i < strlen(filename); i++) {
+							filename[i] = tolower(filename[i]);
+						}
+						// swap extension
+						if (strstr(filename, ".exe")) {
+							strncpy(strstr(filename, ".exe"), ".mpq", 4);
+						} else if (strstr(filename, ".mpq")) {
+							strncpy(strstr(filename, ".mpq"), ".exe", 4);
+						}
+						sprintf(mpqfile, "%s/%s", ArchiveDir, filename);
+					}
+					if (stat(mpqfile, &st)) {
+						// try with initial uppercase
+						filename[0] = toupper(filename[0]);
+						sprintf(mpqfile, "%s/%s", ArchiveDir, filename);
+					}
+					if (stat(mpqfile, &st)) {
+						// try all uppercase
+						for (int i = 0; i < strlen(filename); i++) {
+							filename[i] = toupper(filename[i]);
+						}
+						sprintf(mpqfile, "%s/%s", ArchiveDir, filename);
+					}
+					if (stat(mpqfile, &st)) {
+						sprintf(mpqfile, "%s/%s not found!", ArchiveDir, Todo[u].MPQFile);
+						error(mpqfile, "I also tried different cases and tried both .mpq and .exe extensions\n");
+					}
+					printf("%s from MPQ file \"%s\"\n", Todo[u].File, mpqfile);
 				}
 				sprintf(extract, "%s/%s", Dir, Todo[u].File);
 				if (Todo[u].Arg2 == 1) { // compress
@@ -2789,7 +2873,7 @@ int main(int argc, char** argv)
 	if (!f) {
 		perror("");
 		printf("Can't open %s\n", buf);
-		exit(-1);
+		error("Memory error", "Could not allocate enough memory to read archive.");
 	}
 
 	fprintf(f, "wargus.tales = false\n");
@@ -2817,7 +2901,7 @@ int main(int argc, char** argv)
 	if (!f) {
 		perror("");
 		printf("Can't open %s\n", buf);
-		exit(-1);
+		error("Memory error", "Could not allocate enough memory to read archive.");
 	}
 
 	fputs(VERSION, f);
